#include <iostream>
#include <iomanip>
#include <fstream>
#include <valarray>
#include <vector>
#include <cmath>

#include "TransformMatrix.h"

using namespace std;

int main()
{
	cout << setprecision(2);
	cout.setf(ios_base::fixed);
	TransformMatrix<float> matrix,unity;
	cout << "Matrix nach der Erstellung:"
		 << endl << matrix << endl;
	cout.setf(ios_base::boolalpha);
	cout << "Identical to the unity matrix: " 
	     << (matrix==unity) << endl;
	matrix(2,1)=1;
	cout << "matrix(2,1)=1" << endl 
	     << matrix << endl;
	cout << "Identical to the unity matrix: " 
	     << (matrix==unity) << endl;
	matrix-=unity;
	cout << "matrix-=unity" << endl
		 << matrix << endl;
    matrix+=unity;
	cout << "matrix+=unity" << endl
		 << matrix << endl;
    cout << "matrix*unity=" << endl
    	 << matrix*unity << endl;
    matrix=unity;
    cout << "matrix=unity" << endl
    	 << matrix << endl;
    matrix.translate(3,4,5);	 
    cout << "matrix.translate(3,4,5)" << endl
    	 << matrix << endl;
    float vt[3];
    float v[]={1.0,2.0,3.0};
    valarray<float> vvat(3),vva(v,3);
    vector<float> vvt(3,0),vv(v,v+3);
    cout << "vector v = [ " 
    	 << v[0] << ", " << v[1] << ", " 
    	 << v[2] << "]" << endl;
    matrix.transformQuick(v,vt);
    cout << "C-array: vector v' = [ " 
    	 << vt[0] << ", " << vt[1] << ", " 
    	 << vt[2] << "]" << endl;
    matrix.transformQuick(vva,vvat);
    cout << "valarray: vector v' = [ " 
    	 << vvat[0] << ", " << vvat[1] << ", " 
    	 << vvat[2] << "]" << endl;
    matrix.transformQuick(vv,vvt);
    cout << "std-vector: vector v' = [ " 
    	 << vvt[0] << ", " << vvt[1] << ", " 
    	 << vvat[2] << "]" << endl;
    	 
    v[0]=1; v[1]=0; v[2]=0;
    matrix.reset();
    matrix.rotateAroundZ(M_PI/2.0);
    matrix.transformQuick(v,vt);
    cout << "matrix.reset();" << endl
         << "matrix.rotateAroundZ(M_PI/2.0);" << endl
         << "matrix: " << endl << matrix
         << "vector v = [ " 
    	 << v[0] << ", " << v[1] << ", " 
    	 << v[2] << "]" << endl;
    cout << "vector v' = [ " 
    	 << vt[0] << ", " << vt[1] << ", " 
    	 << vt[2] << "]" << endl;
    
    matrix.reset();
    matrix.rotateAroundY(M_PI/2.0);
    matrix.transformQuick(v,vt);	 
    cout << "matrix.reset();" << endl
         << "matrix.rotateAroundY(M_PI/2.0);" << endl
         << "matrix: " << endl << matrix
         << "vector v = [ " 
    	 << v[0] << ", " << v[1] << ", " 
    	 << v[2] << "]" << endl;
    cout << "vector v' = [ " 
    	 << vt[0] << ", " << vt[1] << ", " 
    	 << vt[2] << "]" << endl << endl;	 
    	 
    v[0]=0; v[1]=0; v[2]=1;
    matrix.reset();
    matrix.rotateAroundX(M_PI/2.0);
    matrix.transformQuick(v,vt);
    cout << "matrix.reset();" << endl
         << "matrix.rotateAroundX(M_PI/2.0);" << endl
         << "matrix: " << endl << matrix
         << "vector v = [ " 
    	 << v[0] << ", " << v[1] << ", " 
    	 << v[2] << "]" << endl;
    cout << "vector v' = [ " 
    	 << vt[0] << ", " << vt[1] << ", " 
    	 << vt[2] << "]" << endl << endl;
    	    
    v[0]=1; v[1]=1; v[2]=1;
    matrix.reset();
    matrix.scale(1,2,3);
    matrix.transformQuick(v,vt);
    cout << "matrix.reset();" << endl
         << "matrix.scale(1,2,3);" << endl
         << "matrix: " << endl << matrix
         << "vector v = [ " 
    	 << v[0] << ", " << v[1] << ", " 
    	 << v[2] << "]" << endl;
    cout << "vector v' = [ " 
    	 << vt[0] << ", " << vt[1] << ", " 
    	 << vt[2] << "]" << endl << endl;
    
    matrix.reset();
    matrix.projectionOrthoBottom();
    cout << "projectionOrthoBottom()"
    	 << endl << matrix << endl << endl;

    matrix.reset();
    matrix.projectionOrthoTop();
    cout << "projectionOrthoTop()"
    	 << endl << matrix << endl << endl;
    	     	     	     
    matrix.reset();
    matrix.projectionOrthoLeft();
    cout << "projectionOrthoLeft()"
    	 << endl << matrix << endl << endl;

    matrix.reset();
    matrix.projectionOrthoRight();
    cout << "projectionOrthoRight()"
    	 << endl << matrix << endl << endl;
    	     	     	     	         	     	     	     	     
    matrix.reset();
    matrix.projectionOnePointPersp(1.0);
    cout << "projectionOnePointPersp(1.0)"
    	 << endl << matrix << endl << endl;
    	    
    matrix.reset();
    matrix.translate(0,0,-5);
    matrix.rotateAroundZ(M_PI/4);
    matrix.rotateAroundX(M_PI/4);
    cout << "matrix: " << endl << matrix;
    TransformMatrix<float> tmp(matrix);
    matrix.invert();
    cout << "inverse: " << endl << matrix
    	 << "matrix*inverse: " << endl
    	 << tmp*matrix << endl;
    	    
    // transform example
    float cubedata[8][3] = {{-1,-1,-1},\
    					 {1,-1,-1},\
    					 {1,1,-1},\
    					 {-1,1,-1},\
    					 {-1,-1,1},\
    					 {1,-1,1},\
    					 {1,1,1},\
    					 {-1,1,1}};
    
    TransformMatrix<float> modelview;
    modelview.rotateAroundX(0*M_PI/180.0);
    modelview.rotateAroundZ(0*M_PI/180.0);
    modelview.translate(0,0,-4);
    
    TransformMatrix<float> topview(modelview);
    topview.projectionOrthoTop();
    TransformMatrix<float> leftview(modelview);
    leftview.projectionOrthoLeft();
    TransformMatrix<float> rightview(modelview);
    rightview.projectionOrthoRight();
    TransformMatrix<float> perspview(modelview);
    perspview.projectionOnePointPersp(0.5);    
    
    ofstream frontviewfile("frontview.txt");
    ofstream leftviewfile("leftview.txt");
    ofstream rightviewfile("rightview.txt");
    ofstream topviewfile("topview.txt");
    ofstream perspviewfile("perspview.txt");
    
    for(size_t k=0;k<8;k++)
    {
    	modelview.transformQuick(cubedata[k],v);
    	frontviewfile << v[0] << " " << v[1] << endl;
    	leftview.transformQuick(cubedata[k],v);
    	leftviewfile << v[0] << " " << v[1] << endl; 
    	rightview.transformQuick(cubedata[k],v);
    	rightviewfile << v[0] << " " << v[1] << endl;
    	topview.transformQuick(cubedata[k],v);
    	topviewfile << v[0] << " " << v[1] << endl;
    	perspview.transform(cubedata[k],v);
    	perspviewfile << v[0] << " " << v[1] << endl;
    }
    cout << "view transform results via gnuplot (e.g. plot \"frontview.txt\")" << endl;
    
  	return 0;
}
